from fastapi_utils.cbv import cbv
from fastapi_utils.inferring_router import InferringRouter

from models.meta import (
    MetaUserModel,
    MetaLogoutModel,
)
from utils.rds import check_instance_is_rds
from services.meta import MetaService


meta_router = InferringRouter(tags=["Meta"])


@cbv(meta_router)
class MetaHandler:
    """
    Class for handling requests for MS RDS
    """

    @meta_router.get("/test")
    def check_rds_instance(self):
        """method for handling server rds check request.

        Returns:
            response (dict): json response
        """

        response = check_instance_is_rds()

        return response

    @meta_router.get("/sessions")
    def rds_sessions(self):
        """method for handling a request to get rds user sessions.

        Returns:
            response (dict): json response
        """

        response = MetaService().get_sessions()

        return response

    @meta_router.post("/login")
    def rds_login(self, params: MetaUserModel):
        """method to handle connection request to rds.

        Args:
            params (RdsUserModel): auth params

        Returns:
            response (dict): json response
        """

        response = MetaService().login(params)

        return response

    @meta_router.post("/testconnect")
    def rds_test_connect(self, params: MetaUserModel):
        """method for handling request for a test connection to RDS.

        Args:
            params (RdsUserModel): auth params

        Returns:
            response (dict): json response
        """

        response = MetaService().connect(params)

        return response

    @meta_router.post("/disconnect")
    def rds_logout_user(self, params: MetaLogoutModel):
        """method for handling a request to disconnect a user.

        Args:
            params (RdsLogoutModel): auth params

        Returns:
            response (dict): json response
        """

        response = MetaService().logout_user(params)

        return response

    @meta_router.get("/apps")
    def rds_apps(self):
        """method for handling a request to get published remote apps.

        Args:
            params (RdsRemoteAppModel): contains collection name

        Returns:
            response (dict): json response
        """

        response = MetaService().get_remote_apps()

        return response
