import winreg as wrg
from contextlib import suppress
from itertools import count

import pythoncom
import wmi
from environment.checks import PlatformEnvironment

# List with proc names for checking rds instance
RDS_PROC_NAMES = ["tssdis.exe"]
# Remote application process alias
REMOTE_APP_PROC_NAME = "rdpinit.exe"
# Message if platform not Windows
NOT_WIN_MESSAGE = "Platform is not Windows!"
# Session states for termidesk Inactive/Active
SESSION_STATES = "IA"


def check_instance_is_rds() -> dict:
    """function returns the answer whether the server is rds.

    Returns:
        response (dict): json response
    """

    result = True
    # Making dict "check_list" through dict comprehension to further
    # check the existence of rds processes
    check_list = {proc_name: False for proc_name in RDS_PROC_NAMES}

    try:
        import psutil

        for process in psutil.process_iter():
            if process.name() in RDS_PROC_NAMES:
                check_list.update({process.name(): True})
        if not all(check_list.values()):
            result = False
        response = dict(is_rds=result, error=None)
    except Exception as ex:
        response = dict(
            is_rds=False,
            error=str(ex) if PlatformEnvironment.is_windows() else NOT_WIN_MESSAGE,
        )

    return response


def win_cli_output_decoder(win_string: str) -> str:
    """function returns a string with normal Cyrillic characters.
    Functionality will be updated.

    Args:
        win_string (string): cp1251 encoded string

    Returns:
        response (string): cp886 encoded string
    """

    return win_string.encode("cp1251").decode("cp866")


def get_remote_apps() -> list:
    """Getting published remote apps using win32 class Win32_RDMSPublishedApplication.

    Args:
        - collection (str): RDS collection alias.

    Returns: list of published remote apps by their collection name.

    """
    location = wrg.HKEY_LOCAL_MACHINE  # реестр локальной машины
    name = r"SOFTWARE\Microsoft\Windows NT\CurrentVersion\Terminal Server\TSAppAllowList\Applications"
    remote_app = []
    with suppress(OSError), wrg.OpenKey(location, name) as access_key:
        remote_app.extend((wrg.EnumKey(access_key, i) for i in count())) 

    return remote_app


class WMIMetaHandler:
    """This class represent a methods for retrieving
    data from published applications using WMI.
    """

    _wmi_handler = None

    def __init__(self):
        if not self._wmi_handler:
            pythoncom.CoInitialize()
            self._wmi_handler = wmi.WMI()

    def get_remote_app_sessions_id(self):
        sessions_id = []
        for process in self._wmi_handler.Win32_Process(Name=REMOTE_APP_PROC_NAME):
            sessions_id.append(process.SessionId)

        return sessions_id
